
% Dependencies 
%   Tensor Toolbox:
%       https://www.tensortoolbox.org/
%   All Real Eigenvectors of Symmetric Tensors (AREST)
%        https://mathweb.ucsd.edu/~njw/CODES/reigsymtensor/areigstsrweb.html
%   Newton Correction Method 
%        https://github.com/arJaffe/NCM

project_loc = "./"

tensor_type = "weighted";
if tensor_type == "weighted"
    output_folder= strcat(project_loc,"weighted_examples/");
elseif tensor_type == "unweighted"
    output_folder = strcat(project_loc,"unweighted_examples/");
end


run_ARTS = true;
run_NCM = true;
run_ONCM = true; 

%
%  Experiment Parameters
%
testing = true; 
if testing 
    ARST_options= struct("Nmax",4,"dt0",.05,"tol",1e-4);
    NCM_trials = 100;
else 
    ARST_options= struct("Nmax",10,"dt0",.05,"tol",1e-4);
    NCM_trials = 5000;
end 


%
%   Generate Tensors 
% 

if testing 
    %[m,n,order,seed]
    examples = {
        [2,2,3,8435], ...
    };
else
    examples = {
        [2,2,3,3333], ...
        [2,2,3,4444], ...
        [2,2,3,956],  ...
        [2,2,3,9250], ...
        [2,2,4,956],  ...
        [2,2,4,3105], ...
        [2,2,4,9250], ...
        [2,2,4,3333], ...
        [2,2,4,3436], ...
        ...
        [2,2,5,5555], ...
        [2,2,5,2321], ...
        [2,2,5,3333], ...
        [2,2,5,2222], ...
        ...
        [3,3,3,1111], ...
        [3,3,3,2222], ...
        ...
        [3,3,4,956],  ...   
        [3,3,4,3105], ...
        [3,3,4,3436], ...    
        [3,3,4,5555], ...
        [3,3,4,9250], ...
        ...
        [3,3,5,1111], ...
        [3,3,5,4444], ...
        ...
        [4,4,3,3333], ...
        [4,4,3,9250], ...
        [4,4,3,956],  ...
        ...
        [4,4,4,1111], ...
        [4,4,4,3436], ...
        [4,4,4,2222], ...
        [4,4,4,9250], ...
        [4,4,4,956],  ...
    };
end


for i =1:length(examples)

    m = examples{i}(1);
    n = examples{i}(2);
    order = examples{i}(3);
    seed = examples{i}(4);

    %
    %  -- Generate/Load Tensor Problems --  % 

    sub_result_folder = strcat(output_folder,"tensor_problems/");
    if ~isdir(sub_result_folder)
        mkdir(sub_result_folder)
    end

    if tensor_type == "weighted"
        mat_file = sprintf("weighted_TKP_n:%d_m:%d_order:%d_seed:%d.mat",n,m,order,seed); 
        filename = strcat(sub_result_folder,mat_file);
    elseif tensor_type == "unweighted"
        mat_file = sprintf("unweighted_TKP_n:%d_m:%d_order:%d_seed:%d.mat",n,m,order,seed);
        filename = strcat(sub_result_folder,mat_file);
    end

    if isfile(filename)
        disp(sprintf("loading:\n %s",mat_file));
        mat_file = load(filename);
        A = mat_file.A;
        B = mat_file.B;
        A_kron_B = mat_file.A_kron_B;
    else
        disp(sprintf("generating:\n %s",mat_file));
        [A,B,A_kron_B] = generate_tensors(n,m,order,seed,tensor_type,sub_result_folder);
    end
    tenParamStr = sprintf("n:%d_m:%d_order:%d_seed:%d",n,m,order,seed);

    %
    %  -- AReigSTensor experiments --  %
    %
    if run_ARTS
        sub_result_folder = strcat(output_folder,"full_spectrum/");
        if ~isdir(sub_result_folder)
            mkdir(sub_result_folder)
        end
        [A_output,B_output,A_kron_B_output,lmd_diff,inner_prod] = AReigSTensor_experiment(A,B,A_kron_B,ARST_options,tenParamStr,sub_result_folder)
        fprintf("AReigST:  (|lambda_A|*|lambda_B| - |lambda_B_kron_A|)/|lambda_B_kron_A|:%f\n",lmd_diff);
        fprintf("AReigST:  inner product :%f\n",inner_prod);
    end 

    %
    %  -- NCM experiments --  %
    %
    if run_NCM
        sub_result_folder = strcat(output_folder,"NCM_sampling/");
        if ~isdir(sub_result_folder)
            mkdir(sub_result_folder)
        end
        method = "NCM";
        [A_output,B_output,A_kron_B_output,lmd_diff,inner_prod] = NCM_experiment(A,B,A_kron_B,seed,NCM_trials,method,tenParamStr,sub_result_folder);
        
        fprintf("NCM:  (|lambda_A|*|lambda_B| - |lambda_B_kron_A|)/|lambda_B_kron_A|:%f\n",lmd_diff);
        fprintf("NCM:  inner product :%f\n",inner_prod);
    end

    if run_ONCM
        sub_result_folder = strcat(output_folder,"ONCM_sampling/");
        if ~isdir(sub_result_folder)
            mkdir(sub_result_folder)
        end
        method = "ONCM";
        [A_output,B_output,A_kron_B_output,lmd_diff,inner_prod] = NCM_experiment(A,B,A_kron_B,seed,NCM_trials,method,tenParamStr,sub_result_folder);
        
        fprintf("ONCM:  (|lambda_A|*|lambda_B| - |lambda_B_kron_A|)/|lambda_B_kron_A|:%f\n",lmd_diff);
        fprintf("ONCM:  inner product :%f\n",inner_prod);
    end
    
end

